﻿'
' DotNetNuke® - http://www.dotnetnuke.com
' Copyright (c) 2002-2008
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Collections.Generic
Imports System.Threading
Imports DotNetNuke.Modules.News.RssToolkit.Rss
Imports System.IO
Imports System.Xml

Namespace DotNetNuke.Modules.News.Business
 ''' <summary>
 ''' Main aggregator class.
 ''' </summary>
 ''' <remarks>
 ''' This class is derived from the RssToolkit aggregator class. 
 ''' It was changed to use our own FeedInfo objects. 
 ''' A new mechanism to identify feeds during download was introduced.
 ''' A new merging method to ensure that feeds without pubDate in items will still get folded into the aggregated feed.
 ''' </remarks>
 ''' <history>
 ''' 	[pdonker]	03/01/2008  Created
 ''' </history>
 Public Class FeedAggregator
  Inherits RssAggregator

#Region " Private Members "
  Private _settings As ModuleSettings
  Private _moduleTitle As String = "New Feed"
  Private _feedErrors As List(Of FeedError)
  Shadows aggregateRss As New Dictionary(Of Integer, String)()
#End Region

#Region " Properties "
  Public Property FeedErrors() As List(Of FeedError)
   Get
    Return _feedErrors
   End Get
   Set(ByVal value As List(Of FeedError))
    _feedErrors = value
   End Set
  End Property
#End Region

#Region " Constructors "
  Public Sub New(ByVal tempDir As String, ByVal NewsSettings As ModuleSettings, ByVal ModuleTitle As String)
   MyBase.New(tempDir)
   _settings = NewsSettings
   _feedErrors = New List(Of FeedError)
   _moduleTitle = ModuleTitle
  End Sub
#End Region

#Region " Public Methods "
  Public Overloads Sub Load(ByVal Feeds As List(Of FeedInfo))

   If Feeds.Count < 1 Then Exit Sub
   rssEvents = New AutoResetEvent(Feeds.Count - 1) {}
   LoadRssFeeds(Feeds)
   MergeRss(_moduleTitle)

  End Sub
#End Region

#Region " Private Methods "
  Private Sub LoadRssFeeds(ByVal Feeds As List(Of FeedInfo))

   For index As Integer = 0 To Feeds.Count - 1
    If rssEvents(index) Is Nothing Then
     rssEvents(index) = New AutoResetEvent(False)
    End If
    Dim feed As FeedInfo = Feeds(index)
    feed.Index = index ' need to keep an index
    ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf GetRssFeedsByFeedInfo), feed)
   Next

   If Thread.CurrentThread.GetApartmentState() = ApartmentState.STA Then
    ' WaitAll for multiple handles on an STA thread is not supported. 
    ' ...so wait on each handle individually. 
    For Each waitHandle As WaitHandle In rssEvents
     waitHandle.WaitAny(New WaitHandle() {waitHandle})
    Next
   Else
    WaitHandle.WaitAll(rssEvents)
   End If
  End Sub

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Private Sub GetRssFeedsByFeedInfo(ByVal state As Object)

   Dim feed As FeedInfo = Nothing


   feed = DirectCast(state, FeedInfo)
   Dim xmlString As String = String.Empty

   ' check to see if we should skip this one
   If feed.FailedRetrieveTimes >= _settings.RetryTimes Then
    If feed.LastRetrieveTry.AddMinutes(_settings.RetryTimeOut) > Now Then
     Exit Sub
    End If
   End If

   Try

    Using rssStream As Stream = DownloadManager.GetFeed(feed)
     Using reader As New XmlTextReader(rssStream)
      While reader.Read()
       If reader.NodeType = XmlNodeType.Element Then
        xmlString = reader.ReadOuterXml()
        Exit While
       End If
      End While
     End Using
    End Using

   Catch ex As Exception
    _feedErrors.Add(New FeedError(feed, "Download error", FeedErrorType.Retrieval))
    rssEvents(feed.Index).[Set]()
    Exit Sub
   End Try

   Try
    ' convert to rss
    Dim originalType As DocumentType = RssToolkit.Rss.DocumentType.Unknown
    xmlString = RssXmlHelper.ConvertToRssXml(xmlString, _tempDir, originalType)

    If Not String.IsNullOrEmpty(xmlString) Then
     Dim validator As New RssXmlSchemaValidator()
     Dim assembly As Reflection.Assembly = Reflection.Assembly.GetExecutingAssembly()
     Using stream As Stream = assembly.GetManifestResourceStream(RssToolkit.Constants.Rss20Xsd)
      SyncLock aggregateRss
       validator.ValidXmlDoc(xmlString, New XmlTextReader(stream))
       If validator.IsValidXml Then
        aggregateRss.Add(feed.FeedID, xmlString)
       Else
        _feedErrors.Add(New FeedError(feed, validator.ValidationError, FeedErrorType.Parsing))
       End If
      End SyncLock
     End Using
    End If
   Catch ex As Exception
    _feedErrors.Add(New FeedError(feed, ex.Message, FeedErrorType.Parsing))
   Finally
    rssEvents(feed.Index).[Set]()
   End Try

  End Sub

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Friend Shadows Sub MergeRss(ByVal title As String)

   LoadXml("<?xml version=""1.0"" encoding=""utf-8""?><rss version=""2.0""><channel></channel></rss>")
   Try

    Dim nodesList As New SortedList(Of String, XmlElement)()
    Dim channelNode As XmlNode = SelectSingleNode("rss/channel")
    Dim feedCount As Integer = 0

    ' First handle Atom and Rss2 feeds (they contain date information on the item level)
    For Each iKey As Integer In aggregateRss.Keys
     Dim feed As FeedInfo = FeedController.GetFeed(iKey, -1)
     Select Case feed.ParsedFeedtype
      Case RssToolkit.Rss.DocumentType.Atom.ToString, RssToolkit.Rss.DocumentType.Rss20.ToString

       feedCount += 1
       Dim inputXml As String = aggregateRss(iKey)
       Dim nodes As XmlNodeList
       Dim rssDocument As New XmlDocument()
       Dim importErrors As Integer = 0
       If Not String.IsNullOrEmpty(inputXml) AndAlso channelNode IsNot Nothing Then
        rssDocument.LoadXml(inputXml)
        nodes = rssDocument.SelectNodes("rss/channel/item")
        Dim itemCnt As Integer = 0
        For Each node As XmlElement In nodes
         itemCnt += 1
         Dim [date] As DateTime = DateTime.MinValue
         Try
          Dim pubDates As XmlNodeList = node.GetElementsByTagName("pubDate")
          If pubDates.Count > 0 Then
           Dim pubDate As XmlNode = pubDates(0)
           [date] = RssXmlHelper.Parse(pubDate.InnerText)
          End If
         Catch ex As Exception
          importErrors += 1
         End Try

         Try
          Dim key As String = ""
          If [date] <> DateTime.MinValue Then
           key = [String].Format(System.Globalization.CultureInfo.InvariantCulture, "{0}_{1}", [date].ToString("u", System.Globalization.CultureInfo.InvariantCulture), node.ChildNodes(0).InnerText)
           Dim iSec As Integer = 0
           Do While nodesList.ContainsKey(key)
            iSec -= 1
            key = [String].Format(System.Globalization.CultureInfo.InvariantCulture, "{0}_{1}", [date].AddSeconds(iSec).ToString("u", System.Globalization.CultureInfo.InvariantCulture), node.ChildNodes(0).InnerText)
           Loop
          Else
           key = feed.FeedID & "-" & itemCnt.ToString
          End If
          If node.SelectSingleNode("pubDate") IsNot Nothing Then
           node.SelectSingleNode("pubDate").InnerText = RssXmlHelper.ToRfc822(RssXmlHelper.Parse(node.SelectSingleNode("pubDate").InnerText))
          Else
           node.AppendChild(DotNetNuke.Common.Utilities.XmlUtils.CreateElement(rssDocument, "pubDate", RssXmlHelper.ToRfc822(Now)))
          End If
          nodesList.Add(key, node)
         Catch ex As Exception
          importErrors += 1
         End Try
        Next

        If importErrors = nodes.Count Then
         _feedErrors.Add(New FeedError(feed, importErrors.ToString & " import errors", FeedErrorType.Aggregation))
        End If

       End If

     End Select
    Next

    ' Add the nodes but insert them to reverse the order to pubDate DESC
    Dim first As Boolean = True
    For Each key As String In nodesList.Keys
     If first Then
      channelNode.AppendChild(ImportNode(nodesList(key), True))
      first = False
     Else
      channelNode.InsertBefore(ImportNode(nodesList(key), True), channelNode.FirstChild)
     End If
    Next

    ' Now handle the other feeds
    For Each iKey As Integer In aggregateRss.Keys
     Dim feed As FeedInfo = FeedController.GetFeed(iKey, -1)
     Select Case feed.ParsedFeedtype
      Case RssToolkit.Rss.DocumentType.Atom.ToString, RssToolkit.Rss.DocumentType.Rss20.ToString
      Case Else

       feedCount += 1
       Dim inputXml As String = aggregateRss(iKey)
       Dim nodes As XmlNodeList
       Dim rssDocument As New XmlDocument()
       Dim importErrors As Integer = 0
       If Not String.IsNullOrEmpty(inputXml) AndAlso channelNode IsNot Nothing Then
        rssDocument.LoadXml(inputXml)
        nodes = rssDocument.SelectNodes("rss/channel/item")
        Dim itemCnt As Integer = 0
        For Each node As XmlElement In nodes
         Dim destination As Integer = itemCnt * (feedCount - 1) + 1 + itemCnt
         Dim destNode As XmlNode = channelNode.ChildNodes.Item(destination)
         If destNode Is Nothing Then
          channelNode.AppendChild(ImportNode(node, True))
         Else
          channelNode.InsertBefore(ImportNode(node, True), destNode)
         End If
         itemCnt += 1
        Next
       End If

     End Select
    Next


    ' Insert the metadata above
    If Not String.IsNullOrEmpty(title) Then
     channelNode.InsertBefore(DotNetNuke.Common.Utilities.XmlUtils.CreateElement(Me, "title", title), channelNode.FirstChild)
    End If

   Catch ex As Exception
    'NotifySubscribers(ex, ex.Message, RssSeverityType.[Error])
   End Try

  End Sub
#End Region

#Region " FeedError "
  Public Structure FeedError
   Public Feed As FeedInfo
   Public ErrorMessage As String
   Public ErrorType As FeedErrorType
   Public Sub New(ByVal feed As FeedInfo, ByVal errorMessage As String, ByVal errorType As FeedErrorType)
    Me.Feed = feed
    Me.ErrorMessage = errorMessage
    Me.ErrorType = errorType
   End Sub
  End Structure

  Public Enum FeedErrorType
   Retrieval
   Aggregation
   Parsing
  End Enum
#End Region

 End Class
End Namespace
