'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.CodeDom
Imports System.CodeDom.Compiler
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Xml
Imports DotNetNuke.Modules.News.RssToolkit.Rss

Namespace DotNetNuke.Modules.News.RssToolkit.Rss.CodeGeneration
	''' <summary>
	''' Used for generating Code using CodeDom
	''' </summary>
	Public NotInheritable Class RssCodeGenerator
		#Region "Constants"
		''' <summary>
		''' Rss Constant
		''' </summary>
		Public Const RssConstant As String = "Rss"

		''' <summary>
		''' Channel Constant
		''' </summary>
		Public Const ChannelConstant As String = "channel"

		''' <summary>
		''' Namespace Constant
		''' </summary>
		Public Const NamespaceConstant As String = "RssToolkit.Rss"

		''' <summary>
		''' Rss Namespace Constant
		''' </summary>
		Public Const RssNamespaceConstant As String = NamespaceConstant + ".RssDocumentBase"

		''' <summary>
		''' Http Handler Namespace Constant
		''' </summary>
		Public Const HttpHandlerNamespaceConstant As String = NamespaceConstant + ".RssHttpHandlerBase"
		#End Region

		#Region "Constructors"
		Private Sub New()
		End Sub
		#End Region

		#Region "Public methods"
		''' <summary>
		''' Generates the code.
		''' </summary>
		''' <param name="rssDefinition">The RSS definition.</param>
		''' <param name="outputLanguage">The output language.</param>
		''' <param name="namespaceName">Name of the namespace.</param>
		''' <param name="classNamePrefix">The class name prefix.</param>
		''' <param name="outputCode">The output code.</param>
		''' <param name="useBaseClass">if set to <c>true</c> [use base class].</param>
		Public Shared Sub GenerateCode(ByVal rssDefinition As RssDocumentBase, ByVal outputLanguage As String, ByVal namespaceName As String, ByVal classNamePrefix As String, ByVal outputCode As TextWriter, ByVal useBaseClass As Boolean)
			If rssDefinition Is Nothing Then
				Throw New ArgumentNullException("rssDefinition")
			End If

			If String.IsNullOrEmpty(outputLanguage) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "outputLanguage"))
			End If

			If namespaceName Is Nothing Then
				Throw New ArgumentNullException("namespaceName")
			End If

			If String.IsNullOrEmpty(classNamePrefix) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "classNamePrefix"))
			End If

			If outputCode Is Nothing Then
				Throw New ArgumentNullException("outputCode")
			End If

   GenerateCode(rssDefinition.ToXml(DocumentType.Rss20), rssDefinition.Url, outputLanguage, namespaceName, classNamePrefix, outputCode, _
    useBaseClass)
		End Sub

		''' <summary>
		''' Generates the code.
		''' </summary>
		''' <param name="codeXml">The code XML.</param>
		''' <param name="url">The URL.</param>
		''' <param name="outputLanguage">The output language.</param>
		''' <param name="namespaceName">Name of the namespace.</param>
		''' <param name="classNamePrefix">The class name prefix.</param>
		''' <param name="outputCode">The output code.</param>
		''' <param name="useBaseClass">if set to <c>true</c> [use base class].</param>
		Public Shared Sub GenerateCode(ByVal codeXml As String, ByVal url As String, ByVal outputLanguage As String, ByVal namespaceName As String, ByVal classNamePrefix As String, ByVal outputCode As TextWriter, _
			ByVal useBaseClass As Boolean)
			If String.IsNullOrEmpty(codeXml) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "codeXml"))
			End If

			If url Is Nothing Then
				Throw New ArgumentNullException("url")
			End If

			If String.IsNullOrEmpty(outputLanguage) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "outputLanguage"))
			End If

			If namespaceName Is Nothing Then
				Throw New ArgumentNullException("namespaceName")
			End If

			If String.IsNullOrEmpty(classNamePrefix) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "classNamePrefix"))
			End If

			If outputCode Is Nothing Then
				Throw New ArgumentNullException("outputCode")
			End If

			' get the CodeDom provider for the language
			Dim provider As CodeDomProvider = CodeDomProvider.CreateProvider(outputLanguage)

			' generate the CodeDom tree
			Dim unit As New CodeCompileUnit()
			GenerateCodeDomTree(codeXml, url, namespaceName, classNamePrefix, unit, useBaseClass)

			' generate source
			Dim options As New CodeGeneratorOptions()
			options.BlankLinesBetweenMembers = True
			options.BracingStyle = "Block"
			options.ElseOnClosing = False
			options.IndentString = "    "

			provider.GenerateCodeFromCompileUnit(unit, outputCode, options)
		End Sub

		''' <summary>
		''' Generates the code DOM tree.
		''' </summary>
		''' <param name="rssDefinition">The RSS definition.</param>
		''' <param name="namespaceName">Name of the namespace.</param>
		''' <param name="classNamePrefix">The class name prefix.</param>
		''' <param name="outputCodeCompileUnit">The output code compile unit.</param>
		''' <param name="useBaseClass">if set to <c>true</c> [use base class].</param>
		Public Shared Sub GenerateCodeDomTree(ByVal rssDefinition As RssDocumentBase, ByVal namespaceName As String, ByVal classNamePrefix As String, ByVal outputCodeCompileUnit As CodeCompileUnit, ByVal useBaseClass As Boolean)
			If rssDefinition Is Nothing Then
				Throw New ArgumentNullException("rssDefinition")
			End If

			If namespaceName Is Nothing Then
				Throw New ArgumentNullException("namespaceName")
			End If

			If String.IsNullOrEmpty(classNamePrefix) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "classNamePrefix"))
			End If

			If outputCodeCompileUnit Is Nothing Then
				Throw New ArgumentNullException("outputCodeCompileUnit")
			End If

   GenerateCodeDomTree(rssDefinition.ToXml(DocumentType.Rss20), rssDefinition.Url, namespaceName, classNamePrefix, outputCodeCompileUnit, useBaseClass)
		End Sub

		''' <summary>
		''' Generates the code DOM tree.
		''' </summary>
		''' <param name="codeXml">The code XML.</param>
		''' <param name="url">The URL.</param>
		''' <param name="namespaceName">Name of the namespace.</param>
		''' <param name="classNamePrefix">The class name prefix.</param>
		''' <param name="outputCodeCompileUnit">The output code compile unit.</param>
		''' <param name="useBaseClass">if set to <c>true</c> [use base class].</param>
		Public Shared Sub GenerateCodeDomTree(ByVal codeXml As String, ByVal url As String, ByVal namespaceName As String, ByVal classNamePrefix As String, ByVal outputCodeCompileUnit As CodeCompileUnit, ByVal useBaseClass As Boolean)
			If String.IsNullOrEmpty(codeXml) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "codeXml"))
			End If

			If namespaceName Is Nothing Then
				Throw New ArgumentNullException("namespaceName")
			End If

			If String.IsNullOrEmpty(classNamePrefix) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "classNamePrefix"))
			End If

			If outputCodeCompileUnit Is Nothing Then
				Throw New ArgumentNullException("outputCodeCompileUnit")
			End If

			If outputCodeCompileUnit.Namespaces Is Nothing Then
				Throw New ArgumentNullException("outputCodeCompileUnit.Namespaces")
			End If

			Dim codeTree As New RssCodeTreeGenerator()
			Dim classDictionary As Dictionary(Of String, ClassInfo) = codeTree.ConvertToDictionary(codeXml)

			' generate namespace
			Dim generatedNamespace As New CodeNamespace(namespaceName)
			generatedNamespace.[Imports].Add(New CodeNamespaceImport("System"))
			generatedNamespace.[Imports].Add(New CodeNamespaceImport("System.Xml"))
			generatedNamespace.[Imports].Add(New CodeNamespaceImport("System.Xml.Serialization"))
			generatedNamespace.[Imports].Add(New CodeNamespaceImport("System.Collections.Generic"))
			generatedNamespace.[Imports].Add(New CodeNamespaceImport(NamespaceConstant))

			For Each classKey As String In classDictionary.Keys
				Dim classInfo As ClassInfo = classDictionary(classKey)
				If classInfo.Properties.Count > 0 Then
					Dim className As String = classNamePrefix + CodeNameFromRssName(classInfo.Name)
					Dim itemType As New CodeTypeDeclaration(className)

					If classInfo.Name.Equals(RssConstant, StringComparison.OrdinalIgnoreCase) Then
						itemType.CustomAttributes.Add(New CodeAttributeDeclaration("System.Serializable"))
						itemType.CustomAttributes.Add(New CodeAttributeDeclaration("System.Xml.Serialization.XmlRoot", New CodeAttributeArgument(New CodePrimitiveExpression("rss"))))

						If useBaseClass Then
							itemType.BaseTypes.Add(New CodeTypeReference(RssNamespaceConstant))
						End If

						If Not String.IsNullOrEmpty(url) Then
							Dim itemTypeName As String = classNamePrefix + "Item"

       '  LoadRss (and LoadRssItems) method
       GenerateLoad(itemType, url)
       GenerateLoadRssByUrl(itemType)
       GenerateLoadRssByXmlTextReader(itemType)
       GenerateLoadRssByXml(itemType)
       GenerateLoadRssItems(itemType, itemTypeName)
      End If

      GenerateToXml(itemType)
      GenerateToDataSet(itemType)
     End If

     If Not NamespaceContainsType(generatedNamespace, itemType.Name) Then
      generatedNamespace.Types.Add(itemType)
     End If

     For Each propertyKey As String In classInfo.Properties.Keys
      Dim [property] As PropertyInfo = classInfo.Properties(propertyKey)

      Dim propertyName As String = classNamePrefix + CodeNameFromRssName([property].Name)
      AddCodeProperty([property].Name, CType(IIf([property].IsAttribute, XmlNodeType.Attribute, XmlNodeType.Element), XmlNodeType), CType(IIf((classDictionary.ContainsKey([property].Name) AndAlso classDictionary([property].Name).Properties.Count > 0), New CodeTypeReference(propertyName), New CodeTypeReference("System.String")), CodeTypeReference), itemType, CBool(IIf([property].Occurrences > 1, True, False)), CStr((IIf(classDictionary.ContainsKey([property].Name), classDictionary([property].Name).[Namespace], String.Empty))))
     Next

     If classInfo.IsText Then
      AddCodeProperty("Text", XmlNodeType.Text, New CodeTypeReference("System.String"), itemType, False, String.Empty)
     End If
    End If
   Next

   If useBaseClass Then
    ' generate http handler base class
    Dim handlerTypeName As String = classNamePrefix + "HttpHandlerBase"
    Dim handlerType As New CodeTypeDeclaration(handlerTypeName)

    handlerType.BaseTypes.Add(New CodeTypeReference(HttpHandlerNamespaceConstant, New CodeTypeReference(0) {New CodeTypeReference(classNamePrefix + RssConstant)}))

    generatedNamespace.Types.Add(handlerType)
   End If

   ' add the generated namespace to the code compile unit
   outputCodeCompileUnit.Namespaces.Add(generatedNamespace)
  End Sub
#End Region

#Region "Private Methods"
  ''' <summary>
  ''' Codes the name of the name from RSS.
  ''' </summary>
  ''' <param name="rssName">Name of the RSS.</param>
  ''' <returns>string</returns>
  Private Shared Function CodeNameFromRssName(ByVal rssName As String) As String
   If String.IsNullOrEmpty(rssName) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "rssName"))
   End If

   Dim sb As New StringBuilder(rssName.Length)

   If rssName.IndexOf(":"c) >= 0 Then
    For Each s As String In rssName.Split(":"c)
     If s.Length > 0 Then
      sb.Append(s.Substring(0, 1).ToUpperInvariant())
     End If

     If s.Length > 1 Then
      sb.Append(s.Substring(1))
     End If
    Next
   Else
    If rssName.Length > 0 Then
     sb.Append(rssName.Substring(0, 1).ToUpperInvariant())
    End If

    If rssName.Length > 1 Then
     sb.Append(rssName.Substring(1))
    End If
   End If

   Return sb.ToString()
  End Function

  ''' <summary>
  ''' Codes the name of the param name from RSS.
  ''' </summary>
  ''' <param name="rssName">Name of the RSS.</param>
  ''' <returns>string</returns>
  Private Shared Function CodeParamNameFromRssName(ByVal rssName As String) As String
   If [String].IsNullOrEmpty(rssName) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "rssName"))
   End If

   Dim name As String = CodeNameFromRssName(rssName)

   If name.Length > 1 Then
    name = name.Substring(0, 1).ToLowerInvariant() + name.Substring(1)
   Else
    name = name.ToLowerInvariant()
   End If

   Return name
  End Function

  ''' <summary>
  ''' Codes the name of the private name from RSS.
  ''' </summary>
  ''' <param name="rssName">Name of the RSS.</param>
  ''' <returns>string</returns>
  Private Shared Function CodePrivateNameFromRssName(ByVal rssName As String) As String
   Return "_" + CodeParamNameFromRssName(rssName)
  End Function

  ''' <summary>
  ''' Namespaces the type of the contains.
  ''' </summary>
  ''' <param name="nameSpace">The nameSpace.</param>
  ''' <param name="typeName">The type name.</param>
  ''' <returns>bool</returns>
  Private Shared Function NamespaceContainsType(ByVal [nameSpace] As CodeNamespace, ByVal typeName As String) As Boolean
   If [nameSpace] Is Nothing Then
    Throw New ArgumentNullException("nameSpace")
   End If

   If String.IsNullOrEmpty(typeName) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "typeName"))
   End If

   For Each type As CodeTypeDeclaration In [nameSpace].Types
    If String.Compare(type.Name, typeName, False, System.Globalization.CultureInfo.InvariantCulture) = 0 Then
     Return True
    End If
   Next

   Return False
  End Function

  ''' <summary>
  ''' Types the contains property.
  ''' </summary>
  ''' <param name="type">The type.</param>
  ''' <param name="propertyToFind">The property to find.</param>
  ''' <returns>bool</returns>
  Private Shared Function TypeContainsProperty(ByVal type As CodeTypeDeclaration, ByVal propertyToFind As String) As Boolean
   If type Is Nothing Then
    Throw New ArgumentNullException("type")
   End If

   If String.IsNullOrEmpty(propertyToFind) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "propertyToFind"))
   End If

   For Each [property] As CodeTypeMember In type.Members
    If String.Compare([property].Name, propertyToFind, False, System.Globalization.CultureInfo.InvariantCulture) = 0 Then
     Return True
    End If
   Next

   Return False
  End Function

  Private Shared Sub AddCodeProperty(ByVal nodeName As String, ByVal nodeType As XmlNodeType, ByVal propertyType As CodeTypeReference, ByVal classType As CodeTypeDeclaration, ByVal collection As Boolean, ByVal nameSpaceName As String)
   Dim propertyVariableName As String = CodePrivateNameFromRssName(nodeName)
   Dim propertyName As String = CodeNameFromRssName(nodeName)

   If collection Then
    propertyVariableName = Pluralizer.ToPlural(propertyVariableName)
    propertyName = Pluralizer.ToPlural(propertyName)
    propertyType = New CodeTypeReference("List", New CodeTypeReference(0) {propertyType})
   End If

   Dim alreadyThere As Boolean = TypeContainsProperty(classType, propertyVariableName)
   If alreadyThere Then
    Return
   End If

   classType.Members.Add(New CodeMemberField(propertyType, propertyVariableName))

   Dim [property] As New CodeMemberProperty()
   [property].Attributes = [property].Attributes And Not MemberAttributes.AccessMask
   [property].Attributes = [property].Attributes Or MemberAttributes.[Public]
   [property].Name = propertyName
   [property].Type = propertyType

   ' prevent warnings for RssDocumentBase properties
   If propertyName.Equals("Version", StringComparison.Ordinal) OrElse propertyName.Equals("Channel", StringComparison.Ordinal) Then
    [property].Attributes = [property].Attributes Or MemberAttributes.[New]
   End If

   Dim cad As CodeAttributeDeclaration
   If nodeType <> XmlNodeType.Text Then
    cad = New CodeAttributeDeclaration("Xml" + nodeType.ToString(), New CodeAttributeArgument(New CodePrimitiveExpression(nodeName)))
   Else
    cad = New CodeAttributeDeclaration("Xml" + nodeType.ToString())
   End If

   If Not [String].IsNullOrEmpty(nameSpaceName) Then
    cad.Arguments.Add(New CodeAttributeArgument("Namespace", New CodePrimitiveExpression(nameSpaceName)))
   End If

   [property].CustomAttributes.Add(cad)

   [property].GetStatements.Add(New CodeMethodReturnStatement(New CodeVariableReferenceExpression(propertyVariableName)))
   [property].SetStatements.Add(New CodeAssignStatement(New CodeVariableReferenceExpression(propertyVariableName), New CodeArgumentReferenceExpression("value")))

   classType.Members.Add([property])
  End Sub

  Private Shared Sub GenerateLoad(ByVal channelType As CodeTypeDeclaration, ByVal url As String)
   Dim varName As String = "doc"
   Dim typeName As String = channelType.Name

   Dim m As New CodeMemberMethod()
   m.Name = "Load"
   m.Attributes = m.Attributes And Not MemberAttributes.AccessMask
   m.Attributes = m.Attributes Or MemberAttributes.[Public] Or MemberAttributes.[Static]
   m.ReturnType = New CodeTypeReference(typeName)

   m.Statements.Add(New CodeVariableDeclarationStatement(typeName, varName))

   m.Statements.Add(New CodeAssignStatement(New CodeVariableReferenceExpression(varName), New CodeMethodInvokeExpression(New CodeVariableReferenceExpression(typeName), "Load", New CodeExpression(0) {New CodeObjectCreateExpression("System.Uri", New CodeExpression(0) {New CodePrimitiveExpression(url)})})))

   m.Statements.Add(New CodeMethodReturnStatement(New CodeVariableReferenceExpression(varName)))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateToXml(ByVal channelType As CodeTypeDeclaration)
   Dim typeName As String = channelType.Name

   Dim m As New CodeMemberMethod()
   m.Name = "ToXml"
   m.Attributes = MemberAttributes.Override Or MemberAttributes.[Public]
   m.Parameters.Add(New CodeParameterDeclarationExpression(GetType(DocumentType), "outputType"))
   m.ReturnType = New CodeTypeReference("System.String")

   m.Statements.Add(New CodeMethodReturnStatement(New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "RssDocumentBase.ToXml", New CodeTypeReference(0) {New CodeTypeReference(typeName)}), New CodeExpression(1) {New CodeArgumentReferenceExpression("outputType"), New CodeThisReferenceExpression()})))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateToDataSet(ByVal channelType As CodeTypeDeclaration)
   Dim m As New CodeMemberMethod()
   m.Name = "ToDataSet"
   m.Attributes = MemberAttributes.[Public]
   m.ReturnType = New CodeTypeReference("System.Data.DataSet")

   m.Statements.Add(New CodeMethodReturnStatement(New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "RssXmlHelper.ToDataSet"), New CodeExpression(0) {New CodeArgumentReferenceExpression("ToXml(DocumentType.Rss)")})))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateLoadRssByUrl(ByVal channelType As CodeTypeDeclaration)
   Dim varName As String = "doc"
   Dim typeName As String = channelType.Name

   Dim m As New CodeMemberMethod()
   m.Name = "Load"
   m.Attributes = m.Attributes And Not MemberAttributes.AccessMask
   m.Attributes = m.Attributes Or MemberAttributes.[Public] Or MemberAttributes.[Static]
   m.Parameters.Add(New CodeParameterDeclarationExpression(GetType(System.Uri), "url"))
   m.ReturnType = New CodeTypeReference(typeName)

   ' Create if Statement
   Dim throwException As New CodeThrowExceptionStatement(New CodeObjectCreateExpression(New CodeTypeReference(GetType(System.ArgumentNullException)), New CodeExpression() {}))

   Dim instanceCondition As New CodeConditionStatement()

   Dim ifCondition As New CodeBinaryOperatorExpression()
   ifCondition.Left = New CodeVariableReferenceExpression("url")
   ifCondition.Right = New CodePrimitiveExpression(Nothing)
   ifCondition.[Operator] = CodeBinaryOperatorType.ValueEquality
   instanceCondition.Condition = ifCondition
   instanceCondition.TrueStatements.Add(throwException)
   m.Statements.Add(instanceCondition)

   m.Statements.Add(New CodeVariableDeclarationStatement(typeName, varName))
   m.Statements.Add(New CodeAssignStatement(New CodeVariableReferenceExpression(varName), New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "RssDocumentBase.Load", New CodeTypeReference(0) {New CodeTypeReference(typeName)}), New CodeExpression(0) {New CodeArgumentReferenceExpression("url")})))

   m.Statements.Add(New CodeMethodReturnStatement(New CodeVariableReferenceExpression(varName)))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateLoadRssByXml(ByVal channelType As CodeTypeDeclaration)
   Dim varName As String = "doc"
   Dim typeName As String = channelType.Name

   Dim m As New CodeMemberMethod()
   m.Name = "Load"
   m.Attributes = m.Attributes And Not MemberAttributes.AccessMask
   m.Attributes = m.Attributes Or MemberAttributes.[Public] Or MemberAttributes.[Static]
   m.Parameters.Add(New CodeParameterDeclarationExpression(GetType(String), "xml"))
   m.ReturnType = New CodeTypeReference(typeName)

   ' Create if Statement
   Dim throwException As New CodeThrowExceptionStatement(New CodeObjectCreateExpression(New CodeTypeReference(GetType(System.ArgumentNullException)), New CodeExpression() {}))

   Dim codecondition As New CodeConditionStatement(New CodeSnippetExpression("String.IsNullOrEmpty(xml)"), throwException)
   m.Statements.Add(codecondition)

   m.Statements.Add(New CodeVariableDeclarationStatement(typeName, varName))
   m.Statements.Add(New CodeAssignStatement(New CodeVariableReferenceExpression(varName), New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "RssDocumentBase.Load", New CodeTypeReference(0) {New CodeTypeReference(typeName)}), New CodeExpression(0) {New CodeArgumentReferenceExpression("xml")})))

   m.Statements.Add(New CodeMethodReturnStatement(New CodeVariableReferenceExpression(varName)))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateLoadRssByXmlTextReader(ByVal channelType As CodeTypeDeclaration)
   Dim varName As String = "doc"
   Dim typeName As String = channelType.Name

   Dim m As New CodeMemberMethod()
   m.Name = "Load"
   m.Attributes = m.Attributes And Not MemberAttributes.AccessMask
   m.Attributes = m.Attributes Or MemberAttributes.[Public] Or MemberAttributes.[Static]
   m.Parameters.Add(New CodeParameterDeclarationExpression(GetType(XmlTextReader), "reader"))
   m.ReturnType = New CodeTypeReference(typeName)

   ' Create if Statement
   Dim throwException As New CodeThrowExceptionStatement(New CodeObjectCreateExpression(New CodeTypeReference(GetType(System.ArgumentNullException)), New CodeExpression() {}))

   Dim instanceCondition As New CodeConditionStatement()

   Dim ifCondition As New CodeBinaryOperatorExpression()
   ifCondition.Left = New CodeVariableReferenceExpression("reader")
   ifCondition.Right = New CodePrimitiveExpression(Nothing)
   ifCondition.[Operator] = CodeBinaryOperatorType.ValueEquality
   instanceCondition.Condition = ifCondition
   instanceCondition.TrueStatements.Add(throwException)
   m.Statements.Add(instanceCondition)
   m.Statements.Add(New CodeVariableDeclarationStatement(typeName, varName))
   m.Statements.Add(New CodeAssignStatement(New CodeVariableReferenceExpression(varName), New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "RssDocumentBase.Load", New CodeTypeReference(0) {New CodeTypeReference(typeName)}), New CodeExpression(0) {New CodeArgumentReferenceExpression("reader")})))

   m.Statements.Add(New CodeMethodReturnStatement(New CodeVariableReferenceExpression(varName)))

   channelType.Members.Add(m)
  End Sub

  Private Shared Sub GenerateLoadRssItems(ByVal channelType As CodeTypeDeclaration, ByVal itemTypeName As String)
   Dim m As New CodeMemberMethod()
   m.Name = "LoadRssItems"
   m.Attributes = m.Attributes And Not MemberAttributes.AccessMask
   m.Attributes = m.Attributes Or MemberAttributes.[Public] Or MemberAttributes.[Static]
   m.ReturnType = New CodeTypeReference("System.Collections.Generic.List", New CodeTypeReference(0) {New CodeTypeReference(itemTypeName)})

   m.Statements.Add(New CodeMethodReturnStatement(New CodePropertyReferenceExpression(New CodeMethodInvokeExpression(New CodeMethodReferenceExpression(Nothing, "Load", Nothing)), "Channel.Items")))

   channelType.Members.Add(m)
  End Sub
#End Region
	End Class
End Namespace
