'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Xml

Namespace DotNetNuke.Modules.News.RssToolkit.Rss.CodeGeneration
	''' <summary>
	''' To generate dictionary used in code generation
	''' </summary>
	Friend Class RssCodeTreeGenerator
		Private ReadOnly stack As New Stack(Of ParentInfo)()

		''' <summary>
		''' Converts to dictionary.
		''' </summary>
		''' <param name="xml">The xml.</param>
		''' <returns>Dictionary</returns>
		Public Function ConvertToDictionary(ByVal xml As String) As Dictionary(Of String, ClassInfo)
			If String.IsNullOrEmpty(xml) Then
				Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "xml"))
			End If

			Using stringReader As New StringReader(xml)
				Using reader As New XmlTextReader(stringReader)
					Return Parse(reader)
				End Using
			End Using
		End Function

		Private Function Parse(ByVal reader As XmlTextReader) As Dictionary(Of String, ClassInfo)
			Dim nps As New XmlNamespaceManager(reader.NameTable)
			Dim table As New Dictionary(Of String, ClassInfo)()
			Dim parent As ParentInfo = Nothing
			Dim previousNode As String = Nothing

			While reader.Read()
				If reader.NodeType = XmlNodeType.Element OrElse reader.NodeType = XmlNodeType.EndElement Then
     ' Got an element. This will be the name of a class.
     Dim className As String = reader.LocalName

     If className.Equals("rss", StringComparison.OrdinalIgnoreCase) Then
      nps = New XmlNamespaceManager(reader.NameTable)
      For index As Integer = 0 To reader.AttributeCount - 1

       reader.MoveToAttribute(index)
       If reader.Name.StartsWith("xmlns") Then
        nps.AddNamespace(reader.LocalName, reader.Value)
       End If
      Next
     End If

     If reader.IsStartElement() OrElse reader.IsEmptyElement Then
      ParseStartElements(reader, table, parent, nps, className)
     Else
      ParseEndElements(table, className)
     End If

     previousNode = className
    ElseIf reader.NodeType = XmlNodeType.Text Then
     If Not String.IsNullOrEmpty(previousNode) Then
      Dim classInfo As ClassInfo = table(previousNode)
      classInfo.IsText = True
     End If
    End If
   End While

   Return table
  End Function

  Private Sub ParseStartElements(ByVal reader As XmlTextReader, ByVal table As Dictionary(Of String, ClassInfo), ByVal parent As ParentInfo, ByVal nps As XmlNamespaceManager, ByVal className As String)
   If stack.Count <> 0 Then
    ' Get the parent
    parent = stack.Peek()

    ' Get the child element from inside the parent
    If parent.ChildCount.ContainsKey(className) Then
     ' increment the Occurrence of the child element inside the parent
     Dim childCount As Integer = parent.ChildCount(className)
     parent.ChildCount(className) = System.Threading.Interlocked.Increment(childCount)
    Else
     ' if child does not exist in the parent add it with Occurrence = 1
     parent.ChildCount.Add(className, 1)
    End If
   End If

   If Not reader.IsEmptyElement Then
    ' If its not an empty element push it in the stack which
    ' contains all parent elements
    stack.Push(New ParentInfo())
   End If

   Dim classInfo As ClassInfo = Nothing

   If Not table.TryGetValue(className, classInfo) Then
    ' Create a ClassInfo every time we hit an element and it is not already
    ' contained in the global table.
    classInfo = New ClassInfo(className)
    table.Add(className, classInfo)
   End If

   If reader.LocalName <> reader.Name Then
    Dim lookupName As String = CStr(IIf(reader.Prefix = "xmlns", reader.LocalName, reader.Prefix))
    classInfo.[Namespace] = nps.LookupNamespace(lookupName)
   End If
   For index As Integer = 0 To reader.AttributeCount - 1

    ' Add any new attributes that are not already defined in the classInfo....
    reader.MoveToAttribute(index)
    Dim localName As String = reader.LocalName
    If Not classInfo.Properties.ContainsKey(localName) AndAlso (Not reader.Name.StartsWith("xmlns")) Then
     Dim propertyInfo As New PropertyInfo(localName, True)
     classInfo.Properties(propertyInfo.Name) = propertyInfo
    End If
   Next
  End Sub

		Private Sub ParseEndElements(ByVal table As Dictionary(Of String, ClassInfo), ByVal className As String)
			Dim parent As ParentInfo = stack.Pop()
			Dim classInfo As ClassInfo = table(className)
			Dim classMembers As Dictionary(Of String, PropertyInfo) = classInfo.Properties

			For Each child As KeyValuePair(Of String, Integer) In parent.ChildCount
				Dim childName As String = child.Key
				Dim childCount As Integer = child.Value
				Dim existingChildCount As Integer = 0

				If classMembers.ContainsKey(childName) Then
					existingChildCount = classMembers(childName).Occurrences
				Else
					Dim [property] As New PropertyInfo(childName, False)
					classMembers(childName) = [property]
				End If

				If childCount > existingChildCount Then
					classMembers(childName).Occurrences = childCount
				End If
			Next
		End Sub

		Private Class ParentInfo
			Private ReadOnly _childCount As New Dictionary(Of String, Integer)()

			''' <summary>
			''' Initializes a new instance of the <see cref="ParentInfo"/> class.
			''' </summary>
			Public Sub New()
			End Sub

			''' <summary>
			''' Gets or sets the child count.
			''' </summary>
			''' <value>The child count.</value>
			Public ReadOnly Property ChildCount() As Dictionary(Of String, Integer)
				Get
					Return _childCount
				End Get
			End Property
		End Class
	End Class
End Namespace
