'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.CodeDom
Imports System.CodeDom.Compiler
Imports System.Collections.Generic
Imports System.Globalization
Imports System.IO
Imports System.Web
Imports System.Web.Compilation
Imports System.Web.Hosting
Imports System.Xml
Imports DotNetNuke.Modules.News.RssToolkit.Rss

Namespace DotNetNuke.Modules.News.RssToolkit.Rss.CodeGeneration
 ''' <summary>
 '''  build provider for .rssdl type - to automatically generate strongly typed
 ''' classes for channels from URLs defined in .rssdl file
 ''' </summary>
 <BuildProviderAppliesTo(BuildProviderAppliesTo.Code)> _
 Public NotInheritable Class RssdlBuildProvider
  Inherits BuildProvider
  ''' <summary>
  ''' Generates source code for the virtual path of the build provider, and adds the source code to a specified assembly builder.
  ''' </summary>
  ''' <param name="assemblyBuilder">The assembly builder that references the source code generated by the build provider.</param>
  ''' <history>
  ''' 	[pdonker]	01/03/2008  Added mechanism to keep track of original feed type
  ''' </history>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId:="System.IO.InvalidDataException.#ctor(System.String)")> _
  Public Overloads Overrides Sub GenerateCode(ByVal assemblyBuilder As AssemblyBuilder)
   If assemblyBuilder Is Nothing Then
    Throw New ArgumentNullException("assemblyBuilder")
   End If

   ' load as XML
   Dim doc As New XmlDocument()
   Using s As Stream = OpenStream(VirtualPath)
    doc.Load(s)
   End Using

   ' validate root rssdl node
   Dim root As XmlNode = doc.DocumentElement

   If root.Name <> "rssdl" Then
    Throw New InvalidDataException(String.Format(CultureInfo.InvariantCulture, "Unexpected root node '{0}' -- expected root 'rssdl' node", root.Name))
   End If

   ' iterate through rss nodes
   Dim n As XmlNode = root.FirstChild
   While n IsNot Nothing
    If n.NodeType <> XmlNodeType.Element Then
     Continue While
    End If

    If n.Name <> "rss" Then
     Throw New InvalidDataException(String.Format(CultureInfo.InvariantCulture, "Unexpected node '{0}' -- expected root 'rss' node", root.Name))
    End If

    Dim file As String = String.Empty
    Dim url As String = String.Empty
    Dim classNamePrefix As String = String.Empty
    Dim namespaceName As String = String.Empty

    For Each attr As XmlAttribute In n.Attributes
     Select Case attr.Name
      Case "name"
       classNamePrefix = attr.Value
       Exit Select
      Case "url"
       url = attr.Value
       Exit Select
      Case "file"
       file = VirtualPathUtility.Combine(VirtualPathUtility.GetDirectory(VirtualPath), attr.Value)
       Exit Select
      Case "namespace"
       namespaceName = attr.Value
       Exit Select
      Case Else
       Throw New InvalidDataException(String.Format(CultureInfo.InvariantCulture, "Unexpected attribute '{0}'", attr.Name))
     End Select
    Next

    If String.IsNullOrEmpty(classNamePrefix) Then
     Throw New InvalidDataException("Missing 'name' attribute")
    End If

    If String.IsNullOrEmpty(url) AndAlso String.IsNullOrEmpty(file) Then
     Throw New InvalidDataException("Missing 'url' or 'file' attribute - one must be specified")
    End If

    If Not String.IsNullOrEmpty(url) AndAlso Not String.IsNullOrEmpty(file) Then
     Throw New InvalidDataException("Only one of 'file' and 'url' can be specified")
    End If

    ' compile channel
    Dim ccu As New CodeCompileUnit()

    If Not String.IsNullOrEmpty(url) Then
     ' load RssDocument
     Using feedStream As Stream = DownloadManager.GetFeed(url, Nothing)
      Using reader As New XmlTextReader(feedStream)
       Dim originalType As DocumentType = RssToolkit.Rss.DocumentType.Unknown
       Dim codeString As String = RssXmlHelper.ConvertToRssXml(reader, Nothing, originalType)
       RssCodeGenerator.GenerateCodeDomTree(codeString, url, namespaceName, classNamePrefix, ccu, True)
      End Using
     End Using
    Else
     Using reader As New XmlTextReader(file)
      Dim originalType As DocumentType = RssToolkit.Rss.DocumentType.Unknown
      Dim codeString As String = RssXmlHelper.ConvertToRssXml(reader, Nothing, originalType)
      RssCodeGenerator.GenerateCodeDomTree(codeString, url, namespaceName, classNamePrefix, ccu, True)
     End Using
    End If

    assemblyBuilder.AddCodeCompileUnit(Me, ccu)
    n = n.NextSibling
   End While
  End Sub
 End Class
End Namespace
