'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Globalization
Imports System.IO
Imports System.Net
Imports System.Text
Imports System.Web
Imports System.Web.Hosting
Imports System.Xml
Imports System.Xml.Serialization
Imports DotNetNuke.Modules.News.RssToolkit.Rss

Namespace DotNetNuke.Modules.News.RssToolkit.Rss
 ''' <summary>
 ''' helper class that provides memory and disk caching of the downloaded feeds
 ''' </summary>
 ''' <history>
 ''' 	[pdonker]	01/03/2008  Caching directory no longer retrieved from web.config but passed as a parameter
 ''' </history>
 Public Class DownloadManager
  Private Shared _downloadManager As New DownloadManager()
  Private ReadOnly _cache As Dictionary(Of String, CacheInfo)
  Private ReadOnly _defaultTtlMinutes As Integer

  Private Sub New()
   ' create in-memory cache
   _cache = New Dictionary(Of String, CacheInfo)()
   _defaultTtlMinutes = 30
  End Sub

  ''' <summary>
  ''' Gets the Feed information.
  ''' </summary>
  ''' <param name="url">The URL.</param>
  ''' <returns>string</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId:="0#")> _
  Public Shared Function GetFeed(ByVal url As String, ByVal tempDir As String) As Stream
   If String.IsNullOrEmpty(url) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "url"))
   End If
   ' _directoryOnDisk = tempDir;
   Return _downloadManager.GetFeedDom(url, tempDir).Data
  End Function

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Private Shared Function GetTempFileNamePrefixFromUrl(ByVal url As String) As String
   Try
    Dim uri As New Uri(url)
    Return String.Format(CultureInfo.InvariantCulture, "{0}_{1:x8}", uri.Host.Replace("."c, "_"c), StableHash(uri.AbsolutePath))
   Catch generatedExceptionName As UriFormatException
    Return "rss"
   End Try
  End Function

  ' returns a hash that is always the same (unlink String.GetHashCode() which varies)
  Public Shared Function StableHash(ByVal value As String) As Integer
   Dim hash As Integer = 1642088727
   ' slurped from .Net runtime internals...
   For Each c As Char In value
    hash <<= 5
    hash = hash Xor AscW(c)
   Next

   Return hash
  End Function

  Private Shared Function GetTtlFromString(ByVal ttlString As String, ByVal defaultTtlMinutes As Integer) As Integer
   If Not String.IsNullOrEmpty(ttlString) Then
    Dim ttlMinutes As Integer
    If Integer.TryParse(ttlString, ttlMinutes) Then
     If ttlMinutes >= 0 Then
      Return ttlMinutes
     End If
    End If
   End If

   Return defaultTtlMinutes
  End Function

  Private Function GetFeedDom(ByVal url As String, ByVal tempDir As String) As CacheInfo
   Dim dom As CacheInfo = Nothing

   SyncLock _cache
    If _cache.TryGetValue(url, dom) Then
     If DateTime.UtcNow > dom.Expiry Then
      TryDeleteFile(dom.FeedFilename)
      _cache.Remove(url)
      dom = Nothing
     End If
    End If
   End SyncLock

   If Not CacheReadable(dom) Then
    dom = DownLoadFeedDom(url, tempDir)

    SyncLock _cache
     _cache(url) = dom
    End SyncLock
   End If

   Return dom
  End Function

  Private Shared Function CacheReadable(ByVal dom As CacheInfo) As Boolean
   Return (dom IsNot Nothing AndAlso dom.Data IsNot Nothing AndAlso dom.Data.CanRead)
  End Function

  Private Function DownLoadFeedDom(ByVal url As String, ByVal tempDir As String) As CacheInfo
   '  look for disk cache first
   Dim dom As CacheInfo = TryLoadFromDisk(url, tempDir)

   If CacheReadable(dom) Then
    Return dom
   End If

   Dim ttlString As String = Nothing
   Dim doc As New XmlDocument()
   doc.Load(url)

   If doc.SelectSingleNode("/rss/channel/ttl") IsNot Nothing Then
    ttlString = doc.SelectSingleNode("/rss/channel/ttl").Value
   End If

   '  compute expiry
   Dim ttlMinutes As Integer = GetTtlFromString(ttlString, _defaultTtlMinutes)
   Dim utcExpiry As DateTime = DateTime.UtcNow.AddMinutes(ttlMinutes)

   '  save to disk
   Return TrySaveToDisk(doc, url, utcExpiry, tempDir)
  End Function

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Private Function TryLoadFromDisk(ByVal url As String, ByVal tempDir As String) As CacheInfo
   If tempDir Is Nothing Then
    ' no place to cache
    Return Nothing
   End If

   Dim found As CacheInfo = Nothing

   ' look for all files matching the prefix
   ' looking for the one matching url that is not expired
   ' removing expired (or invalid) ones
   Dim pattern As String = GetTempFileNamePrefixFromUrl(url) + "_*.feed"
   Dim files As String() = Directory.GetFiles(tempDir, pattern, SearchOption.TopDirectoryOnly)

   For Each feedFilename As String In files
    Dim feedDoc As XmlDocument = Nothing
    Dim isFeedFileValid As Boolean = False
    Dim utcExpiryFromFeedFile As DateTime = DateTime.MinValue
    Dim urlFromFeedFile As String = Nothing

    Try
     feedDoc = New XmlDocument()
     feedDoc.Load(feedFilename)

     ' look for special XML comment (before the root tag)'
     ' containing expiration and url
     Dim comment As XmlComment = TryCast(feedDoc.DocumentElement.PreviousSibling, XmlComment)

     If comment IsNot Nothing Then
      Dim c As String = CStr(IIf(comment.Value Is Nothing, String.Empty, comment.Value))
      Dim i As Integer = c.IndexOf("@"c)
      Dim expiry As Long

      If i >= 0 AndAlso Long.TryParse(c.Substring(0, i), expiry) Then
       utcExpiryFromFeedFile = DateTime.FromBinary(expiry)
       urlFromFeedFile = c.Substring(i + 1)
       isFeedFileValid = True
      End If
     End If
     ' error processing one file shouldn't stop processing other files
    Catch generatedExceptionName As XmlException
    End Try

    ' remove invalid or expired file
    If Not isFeedFileValid OrElse utcExpiryFromFeedFile < DateTime.UtcNow Then
     TryDeleteFile(feedFilename)
     ' try next file
     Continue For
    End If

    ' match url
    If urlFromFeedFile = url Then
     ' keep the one that expires last
     If found Is Nothing OrElse found.Expiry < utcExpiryFromFeedFile Then
      ' if we have a previously found older expiration, kill it...
      If found IsNot Nothing Then
       found.Dispose()
       TryDeleteFile(found.FeedFilename)
      End If

      ' create DOM and set expiry (as found on disk)
      found = New CacheInfo(feedDoc, utcExpiryFromFeedFile, feedFilename)
     End If
    End If
   Next

   ' return best fit
   Return found
  End Function

  Private Sub TryDeleteFile(ByVal feedFilename As String)
   If String.IsNullOrEmpty(feedFilename) Then
    Return
   End If

   Try
    File.Delete(feedFilename)
    ' error deleting a file shouldn't stop processing other files
   Catch
   End Try
  End Sub

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Private Function TrySaveToDisk(ByVal doc As XmlDocument, ByVal url As String, ByVal utcExpiry As DateTime, ByVal tempDir As String) As CacheInfo
   Dim fileName As String = Nothing

   If tempDir IsNot Nothing Then
    Dim comment As XmlComment = doc.CreateComment(String.Format(CultureInfo.InvariantCulture, "{0}@{1}", utcExpiry.ToBinary(), url))
    doc.InsertBefore(comment, doc.DocumentElement)

    fileName = String.Format(CultureInfo.InvariantCulture, "{0}_{1:x8}.feed", GetTempFileNamePrefixFromUrl(url), Guid.NewGuid().GetHashCode())

    Try
     doc.Save(Path.Combine(tempDir, fileName))
    Catch
     ' can't save to disk - not a problem   
     fileName = Nothing
    End Try
   End If

   Return New CacheInfo(doc, utcExpiry, fileName)
  End Function

  Private Class CacheInfo
   Implements IDisposable
   Private ReadOnly m_data As Stream
   Private ReadOnly m_expiry As DateTime
   Private ReadOnly filename As String

   Public Sub New(ByVal doc As XmlDocument, ByVal utcExpiry As DateTime, ByVal feedFilename As String)
    Dim documentStream As New MemoryStream()
    doc.Save(documentStream)
    documentStream.Flush()
    documentStream.Position = 0
    Me.m_data = documentStream
    Me.m_expiry = utcExpiry
    Me.filename = feedFilename
   End Sub

   ''' <summary>
   ''' Gets the expiration time in UTC.
   ''' </summary>
   ''' <value>The expiry.</value>
   Public ReadOnly Property Expiry() As DateTime
    Get
     Return m_expiry
    End Get
   End Property

   ''' <summary>
   ''' Gets the data stream
   ''' </summary>
   ''' <value>The data.</value>
   Public ReadOnly Property Data() As Stream
    Get
     Return m_data
    End Get
   End Property

   ''' <summary>
   ''' Gets the filename 
   ''' </summary>
   Public ReadOnly Property FeedFilename() As String
    Get
     Return filename
    End Get
   End Property

   ''' <summary>
   ''' Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
   ''' </summary>
   Public Sub Dispose() Implements IDisposable.Dispose
    m_data.Dispose()
   End Sub
  End Class
 End Class
End Namespace
