'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.Collections.Generic
Imports System.Globalization
Imports System.Text

Namespace DotNetNuke.Modules.News.RssToolkit
 ''' <summary>
 ''' Used to convert to Plural or Singular
 ''' </summary>
 Public NotInheritable Class Pluralizer
  Private Sub New()
  End Sub
  Shared _specialSingulars As Dictionary(Of String, Word)
  Shared _specialPlurals As Dictionary(Of String, Word)
  Shared _suffixRules As List(Of SuffixRule)

#Region "Special Words Table"
  Private Shared _specialWordsStringTable As String() = New String() {"agendum", "agenda", "", "albino", "albinos", "", _
   "alga", "algae", "", "alumna", "alumnae", "", _
   "apex", "apices", "apexes", "archipelago", "archipelagos", "", _
   "bacterium", "bacteria", "", "beef", "beefs", "beeves", _
   "bison", "", "", "brother", "brothers", "brethren", _
   "candelabrum", "candelabra", "", "carp", "", "", _
   "casino", "casinos", "", "child", "children", "", _
   "chassis", "", "", "chinese", "", "", _
   "clippers", "", "", "cod", "", "", _
   "codex", "codices", "", "commando", "commandos", "", _
   "corps", "", "", "cortex", "cortices", "cortexes", _
   "cow", "cows", "kine", "criterion", "criteria", "", _
   "datum", "data", "", "debris", "", "", _
   "diabetes", "", "", "ditto", "dittos", "", _
   "djinn", "", "", "dynamo", "", "", _
   "elk", "", "", "embryo", "embryos", "", _
   "ephemeris", "ephemeris", "ephemerides", "erratum", "errata", "", _
   "extremum", "extrema", "", "fiasco", "fiascos", "", _
   "fish", "fishes", "fish", "flounder", "", "", _
   "focus", "focuses", "foci", "fungus", "fungi", "funguses", _
   "gallows", "", "", "genie", "genies", "genii", _
   "ghetto", "ghettos", "", "graffiti", "", "", _
   "headquarters", "", "", "herpes", "", "", _
   "homework", "", "", "index", "indices", "indexes", _
   "inferno", "infernos", "", "japanese", "", "", _
   "jumbo", "jumbos", "", "latex", "latices", "latexes", _
   "lingo", "lingos", "", "mackerel", "", "", _
   "macro", "macros", "", "manifesto", "manifestos", "", _
   "measles", "", "", "money", "moneys", "monies", _
   "mongoose", "mongooses", "mongoose", "mumps", "", "", _
   "murex", "murecis", "", "mythos", "mythos", "mythoi", _
   "news", "", "", "octopus", "octopuses", "octopodes", _
   "ovum", "ova", "", "ox", "ox", "oxen", _
   "photo", "photos", "", "pincers", "", "", _
   "pliers", "", "", "pro", "pros", "", _
   "rabies", "", "", "radius", "radiuses", "radii", _
   "rhino", "rhinos", "", "salmon", "", "", _
   "scissors", "", "", "series", "", "", _
   "shears", "", "", "silex", "silices", "", _
   "simplex", "simplices", "simplexes", "soliloquy", "soliloquies", "soliloquy", _
   "species", "", "", "stratum", "strata", "", _
   "swine", "", "", "trout", "", "", _
   "tuna", "", "", "vertebra", "vertebrae", "", _
   "vertex", "vertices", "vertexes", "vortex", "vortices", "vortexes"}
#End Region

#Region "Suffix Rules Table"
  Private Shared _suffixRulesStringTable As String() = New String() {"ch", "ches", "sh", "shes", "ss", "sses", _
   "ay", "ays", "ey", "eys", "iy", "iys", _
   "oy", "oys", "uy", "uys", "y", "ies", _
   "ao", "aos", "eo", "eos", "io", "ios", _
   "oo", "oos", "uo", "uos", "o", "oes", _
   "cis", "ces", "sis", "ses", "xis", "xes", _
   "louse", "lice", "mouse", "mice", "zoon", "zoa", _
   "man", "men", "deer", "deer", "fish", "fish", _
   "sheep", "sheep", "itis", "itis", "ois", "ois", _
   "pox", "pox", "ox", "oxes", "foot", "feet", _
   "goose", "geese", "tooth", "teeth", "alf", "alves", _
   "elf", "elves", "olf", "olves", "arf", "arves", _
   "leaf", "leaves", "nife", "nives", "life", "lives", _
   "wife", "wives"}
#End Region

#Region "public APIs"
  ''' <summary>
  ''' Initializes the <see cref="Pluralizer"/> class.
  ''' </summary>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1810:InitializeReferenceTypeStaticFieldsInline")> _
  Shared Sub New()
   ' populate lookup tables for special words
   _specialSingulars = New Dictionary(Of String, Word)(StringComparer.OrdinalIgnoreCase)
   _specialPlurals = New Dictionary(Of String, Word)(StringComparer.OrdinalIgnoreCase)
   For i As Integer = 0 To _specialWordsStringTable.Length - 1 Step 3

    Dim s As String = _specialWordsStringTable(i)
    Dim p As String = _specialWordsStringTable(i + 1)
    Dim p2 As String = _specialWordsStringTable(i + 2)

    If String.IsNullOrEmpty(p) Then
     p = s
    End If

    Dim w As New Word(s, p)

    _specialSingulars.Add(s, w)
    _specialPlurals.Add(p, w)

    If Not String.IsNullOrEmpty(p2) Then
     _specialPlurals.Add(p2, w)
    End If
   Next

   ' populate suffix rules list
   _suffixRules = New List(Of SuffixRule)()
   For i As Integer = 0 To _suffixRulesStringTable.Length - 1 Step 2

    Dim singular As String = _suffixRulesStringTable(i)
    Dim plural As String = _suffixRulesStringTable(i + 1)
    _suffixRules.Add(New SuffixRule(singular, plural))
   Next
  End Sub

  ''' <summary>
  ''' Converts to the plural.
  ''' </summary>
  ''' <param name="noun">The word.</param>
  ''' <returns>plural form of the word</returns>
  Public Shared Function ToPlural(ByVal noun As String) As String
   Return AdjustCase(ToPluralInternal(noun), noun)
  End Function

  ''' <summary>
  ''' Converts to the singular.
  ''' </summary>
  ''' <param name="noun">The word.</param>
  ''' <returns>singular form of the word</returns>
  Public Shared Function ToSingular(ByVal noun As String) As String
   Return AdjustCase(ToSingularInternal(noun), noun)
  End Function

  ''' <summary>
  ''' Determines whether [is noun plural of noun] [the specified plural].
  ''' </summary>
  ''' <param name="plural">The plural.</param>
  ''' <param name="singular">The singular.</param>
  ''' <returns>
  ''' 	<c>true</c> if [is noun plural of noun] [the specified plural]; otherwise, <c>false</c>.
  ''' </returns>
  Public Shared Function IsNounPluralOfNoun(ByVal plural As String, ByVal singular As String) As Boolean
   Return [String].Compare(ToSingularInternal(plural), singular, StringComparison.OrdinalIgnoreCase) = 0
  End Function
#End Region

#Region "Implementation Details"
  Private Shared Function ToPluralInternal(ByVal s As String) As String
   If String.IsNullOrEmpty(s) Then
    Return s
   End If

   ' lookup special words
   Dim word As Word = Nothing

   If _specialSingulars.TryGetValue(s, word) Then
    Return word.Plural
   End If

   ' apply suffix rules
   For Each rule As SuffixRule In _suffixRules
    Dim plural As String = Nothing
    If rule.TryToPlural(s, plural) Then
     Return plural
    End If
   Next

   ' apply the default rule
   Return s + "s"
  End Function

  Private Shared Function ToSingularInternal(ByVal s As String) As String
   If String.IsNullOrEmpty(s) Then
    Return s
   End If

   ' lookup special words
   Dim word As Word = Nothing

   If _specialPlurals.TryGetValue(s, word) Then
    Return word.Singular
   End If

   ' apply suffix rules
   For Each rule As SuffixRule In _suffixRules
    Dim singular As String = Nothing
    If rule.TryToSingular(s, singular) Then
     Return singular
    End If
   Next

   ' apply the default rule
   If s.EndsWith("s", StringComparison.OrdinalIgnoreCase) Then
    Return s.Substring(0, s.Length - 1)
   End If

   Return s
  End Function

  Private Shared Function AdjustCase(ByVal s As String, ByVal template As String) As String
   If String.IsNullOrEmpty(s) Then
    Return s
   End If

   ' determine the type of casing of the template string
   Dim foundUpperOrLower As Boolean = False
   Dim allLower As Boolean = True
   Dim allUpper As Boolean = True
   Dim firstUpper As Boolean = False
   For i As Integer = 0 To template.Length - 1

    If [Char].IsUpper(template(i)) Then
     If i = 0 Then
      firstUpper = True
     End If
     allLower = False
     foundUpperOrLower = True
    ElseIf [Char].IsLower(template(i)) Then
     allUpper = False
     foundUpperOrLower = True
    End If
   Next

   ' change the case according to template
   If foundUpperOrLower Then
    If allLower Then
     s = s.ToLowerInvariant()
    ElseIf allUpper Then
     s = s.ToUpperInvariant()
    ElseIf firstUpper Then
     If Not [Char].IsUpper(s(0)) Then
      s = s.Substring(0, 1).ToUpperInvariant() + s.Substring(1)
     End If
    End If
   End If

   Return s
  End Function
#End Region

  ''' <summary>
  ''' Suffix rule
  ''' </summary>
  Private Class SuffixRule
   Private ReadOnly _singularSuffix As String
   Private ReadOnly _pluralSuffix As String

   ''' <summary>
   ''' Initializes a new instance of the <see cref="SuffixRule"/> class.
   ''' </summary>
   ''' <param name="singular">The singular.</param>
   ''' <param name="plural">The plural.</param>
   Public Sub New(ByVal singular As String, ByVal plural As String)
    If String.IsNullOrEmpty(singular) Then
     Throw New ArgumentException(String.Format(DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.Culture, DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.ArgmentException, "singular"))
    End If

    If String.IsNullOrEmpty(plural) Then
     Throw New ArgumentException(String.Format(DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.Culture, DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.ArgmentException, "plural"))
    End If

    _singularSuffix = singular
    _pluralSuffix = plural
   End Sub

   ''' <summary>
   ''' Tries to plural.
   ''' </summary>
   ''' <param name="word">The word.</param>
   ''' <param name="plural">The plural.</param>
   ''' <returns>bool</returns>
   Public Function TryToPlural(ByVal word As String, ByRef plural As String) As Boolean
    If String.IsNullOrEmpty(word) Then
     Throw New ArgumentException(String.Format(DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.Culture, DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.ArgmentException, "word"))
    End If

    If word.EndsWith(_singularSuffix, StringComparison.OrdinalIgnoreCase) Then
     plural = word.Substring(0, word.Length - _singularSuffix.Length) + _pluralSuffix
     Return True
    Else
     plural = Nothing
     Return False
    End If
   End Function

   ''' <summary>
   ''' Tries to singular.
   ''' </summary>
   ''' <param name="word">The word.</param>
   ''' <param name="singular">The singular.</param>
   ''' <returns>bool</returns>
   Public Function TryToSingular(ByVal word As String, ByRef singular As String) As Boolean
    If String.IsNullOrEmpty(word) Then
     Throw New ArgumentException(String.Format(DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.Culture, DotNetNuke.Modules.News.RssToolkit.Rss.Resources.RssToolkit.ArgmentException, "word"))
    End If

    If word.EndsWith(_pluralSuffix, StringComparison.OrdinalIgnoreCase) Then
     singular = word.Substring(0, word.Length - _pluralSuffix.Length) + _singularSuffix
     Return True
    Else
     singular = Nothing
     Return False
    End If
   End Function
  End Class

  ''' <summary>
  ''' Word
  ''' </summary>
  Private Class Word
   ''' <summary>
   ''' String representing Singular
   ''' </summary>
   Friend ReadOnly Singular As String

   ''' <summary>
   ''' String representing Plural
   ''' </summary>
   Friend ReadOnly Plural As String

   ''' <summary>
   ''' Initializes a new instance of the <see cref="Word"/> class.
   ''' </summary>
   ''' <param name="singular">The singular.</param>
   ''' <param name="plural">The plural.</param>
   Public Sub New(ByVal singular As String, ByVal plural As String)
    If singular Is Nothing Then
     Throw New ArgumentNullException("singular")
    End If

    If plural Is Nothing Then
     Throw New ArgumentNullException("plural")
    End If

    singular = singular
    plural = plural
   End Sub
  End Class
 End Class
End Namespace
