'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.CodeDom
Imports System.CodeDom.Compiler
Imports System.Collections.Generic
Imports System.Globalization
Imports System.IO
Imports System.Net
Imports System.Reflection
Imports System.Text
Imports System.Threading
Imports System.Web
Imports System.Web.Compilation
Imports System.Web.Hosting
Imports System.Xml
Imports System.Xml.XPath
Imports DotNetNuke.Modules.News.RssToolkit.Opml
Imports DotNetNuke.Modules.News.Business

Namespace DotNetNuke.Modules.News.RssToolkit.Rss
 ''' <summary>
 ''' Used for Aggregating Rss links inside an OPML file
 ''' </summary>
 ''' <history>
 ''' 	[pdonker]	01/03/2008  Caching directory no longer retrieved from web.config but passed as a parameter
 ''' 	[pdonker]	01/03/2008  Several repairs to original code to make it more robust
 ''' </history>
 Public Class RssAggregator
  Inherits XmlDocument

  Friend rssEvents As AutoResetEvent()
  Friend aggregateRss As New List(Of String)()
  Friend _tempDir As String

  ''' <summary>
  ''' Initializes a new instance of the <see cref="RssAggregator"/> class.
  ''' </summary>
  Public Sub New(ByVal tempDir As String)
   MyBase.New()
   _tempDir = tempDir
  End Sub

  ''' <summary>
  ''' Event Handler for Aggregation errors
  ''' </summary>
  Public Event RssAggregationEvent As EventHandler(Of RssAggregationEventArgs)

  ''' <summary>
  ''' Loads from URL.
  ''' </summary>
  ''' <param name="opmlUrl">The opml URL.</param>
  Public Overloads Sub Load(ByVal opmlUrl As System.Uri)
   If opmlUrl Is Nothing Then
    Throw New ArgumentNullException("opmlUrl")
   End If

   Dim opmlDoc As OpmlDocument = OpmlDocument.Load(opmlUrl, _tempDir)

   Load(opmlDoc)
  End Sub

  ''' <summary>
  ''' Loads from XML.
  ''' </summary>
  ''' <param name="xml">The XML.</param>
  Public Overrides Sub Load(ByVal xml As String)
   If String.IsNullOrEmpty(xml) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "xml"))
   End If

   Dim opmlDoc As OpmlDocument = OpmlDocument.Load(xml)

   Load(opmlDoc)
  End Sub

  Private Overloads Sub Load(ByVal opmlDocument As OpmlDocument)
   rssEvents = New AutoResetEvent(opmlDocument.Body.Outlines.Count - 1) {}

   LoadRssFeeds(opmlDocument)

   MergeRss(opmlDocument.Head.Title)
  End Sub


  Private Sub LoadRssFeeds(ByVal opmlDocument As OpmlDocument)
   Dim outlines As IList(Of OpmlOutline) = opmlDocument.Body.Outlines
   For index As Integer = 0 To outlines.Count - 1
    If rssEvents(index) Is Nothing Then
     rssEvents(index) = New AutoResetEvent(False)
    End If

    ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf GetRssFeeds), New OutlineInfo(outlines(index), index))
   Next

   If Thread.CurrentThread.GetApartmentState() = ApartmentState.STA Then
    ' WaitAll for multiple handles on an STA thread is not supported. 
    ' ...so wait on each handle individually. 
    For Each waitHandle As WaitHandle In rssEvents
     waitHandle.WaitAny(New WaitHandle() {waitHandle})
    Next
   Else
    WaitHandle.WaitAll(rssEvents)
   End If
  End Sub

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Private Sub GetRssFeeds(ByVal state As Object)

   Dim outline As OutlineInfo = Nothing

   Try
    outline = DirectCast(state, OutlineInfo)
    Dim xmlString As String = String.Empty

    Using rssStream As Stream = DownloadManager.GetFeed(outline.Outline.XmlUrl, _tempDir)
     Using reader As New XmlTextReader(rssStream)
      While reader.Read()
       If reader.NodeType = XmlNodeType.Element Then
        xmlString = reader.ReadOuterXml()
        Exit While
       End If
      End While
     End Using
    End Using

    ' convert to rss
    Dim originalType As DocumentType = RssToolkit.Rss.DocumentType.Unknown
    xmlString = RssXmlHelper.ConvertToRssXml(xmlString, _tempDir, originalType)

    If Not String.IsNullOrEmpty(xmlString) Then
     Dim validator As New RssXmlSchemaValidator()
     Dim assembly As Assembly = assembly.GetExecutingAssembly()
     Using stream As Stream = assembly.GetManifestResourceStream(Constants.Rss20Xsd)
      SyncLock aggregateRss
       validator.ValidXmlDoc(xmlString, New XmlTextReader(stream))

       If validator.IsValidXml Then
        aggregateRss.Add(xmlString)
       End If
      End SyncLock

      If Not validator.IsValidXml Then
       NotifySubscribers(Nothing, "Rss not valid in OPML for - " + outline.Outline.XmlUrl, RssSeverityType.Warning)
      End If
     End Using
    End If
   Catch ex As Exception
    NotifySubscribers(ex, ex.Message, RssSeverityType.[Error])
   Finally
    rssEvents(outline.Index).[Set]()
   End Try
  End Sub

  Private Sub NotifySubscribers(ByVal ex As Exception, ByVal message As String, ByVal severityType As RssSeverityType)
   Dim eventArgs As New RssAggregationEventArgs()
   eventArgs.Exception = ex
   eventArgs.Message = message
   eventArgs.SeverityType = severityType

   'If RssAggregationEvent IsNot Nothing Then
   RaiseEvent RssAggregationEvent(Me, eventArgs)
   'End If
  End Sub

  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
  Friend Sub MergeRss(ByVal title As String)

   LoadXml("<?xml version=""1.0"" encoding=""utf-8""?><rss version=""2.0""><channel></channel></rss>")
   Try

    Dim nodesList As New SortedList(Of String, XmlElement)()
    Dim channelNode As XmlNode = SelectSingleNode("rss/channel")

    For Each inputXml As String In aggregateRss
     Dim nodes As XmlNodeList
     Dim rssDocument As New XmlDocument()
     If Not String.IsNullOrEmpty(inputXml) AndAlso channelNode IsNot Nothing Then
      rssDocument.LoadXml(inputXml)
      nodes = rssDocument.SelectNodes("rss/channel/item")
      For Each node As XmlElement In nodes
       Dim [date] As DateTime = DateTime.MinValue
       Try
        Dim pubDates As XmlNodeList = node.GetElementsByTagName("pubDate")
        If pubDates.Count > 0 Then
         Dim pubDate As XmlNode = pubDates(0)
         [date] = RssXmlHelper.Parse(pubDate.InnerText)
        End If
       Catch ex As Exception
        NotifySubscribers(ex, ex.Message, RssSeverityType.Warning)
       End Try

       ' add to list, it will sort by the date
       Try
        Dim key As String = [String].Format(System.Globalization.CultureInfo.InvariantCulture, "{0}_{1}", [date].ToString("u", System.Globalization.CultureInfo.InvariantCulture), node.ChildNodes(0).InnerText)
        'node.SelectSingleNode("pubDate").InnerText = RssXmlHelper.Parse(node.SelectSingleNode("pubDate").InnerText).ToString("u", System.Globalization.CultureInfo.InvariantCulture)
        node.SelectSingleNode("pubDate").InnerText = RssXmlHelper.ToRfc822(RssXmlHelper.Parse(node.SelectSingleNode("pubDate").InnerText))
        nodesList.Add(key, node)
       Catch argumentException As ArgumentException
        NotifySubscribers(argumentException, argumentException.Message, RssSeverityType.Warning)
       End Try
      Next

     End If
    Next

    ' Add the nodes but insert them to reverse the order to pubDate DESC
    Dim first As Boolean = True
    For Each key As String In nodesList.Keys
     If first Then
      channelNode.AppendChild(ImportNode(nodesList(key), True))
      first = False
     Else
      channelNode.InsertBefore(ImportNode(nodesList(key), True), channelNode.FirstChild)
     End If
    Next

    ' Insert the metadata above
    If Not String.IsNullOrEmpty(title) Then
     channelNode.InsertBefore(DotNetNuke.Common.Utilities.XmlUtils.CreateElement(Me, "title", title), channelNode.FirstChild)
    End If

   Catch ex As Exception
    NotifySubscribers(ex, ex.Message, RssSeverityType.[Error])
   End Try

  End Sub
 End Class
End Namespace
