'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.Data
Imports System.Collections.Generic
Imports System.IO
Imports System.Reflection
Imports System.Text
Imports System.Xml
Imports System.Xml.Serialization

Namespace DotNetNuke.Modules.News.RssToolkit.Rss
 ''' <summary>
 ''' Abstract class for RSS Document
 ''' </summary>
 ''' <history>
 ''' 	[pdonker]	01/03/2008  Caching directory no longer retrieved from web.config but passed as a parameter
 ''' </history>
 Public MustInherit Class RssDocumentBase
  Private _url As String
  Private _version As String
  Private _channel As RssChannel

  ''' <summary>
  ''' Gets or sets the version.
  ''' </summary>
  ''' <value>The version.</value>
  <XmlAttribute("version")> _
  Public Property Version() As String
   Get
    Return _version
   End Get

   Set(ByVal value As String)
    _version = value
   End Set
  End Property

  ''' <summary>
  ''' Channel
  ''' </summary>
  <XmlIgnore()> _
  Public Property Channel() As RssChannel
   Get
    Return _channel
   End Get

   Set(ByVal value As RssChannel)
    _channel = value
   End Set
  End Property

  ''' <summary>
  ''' Gets or sets the URL.
  ''' </summary>
  ''' <value>The URL.</value>
  Friend Property Url() As String
   Get
    Return _url
   End Get

   Set(ByVal value As String)
    _url = value
   End Set
  End Property

  ''' <summary>
  ''' Returns Xml in the type specified by outputType
  ''' </summary>
  ''' <param name="outputType">Type of the output.</param>
  ''' <returns></returns>
  Public Overridable Function ToXml(ByVal outputType As DocumentType) As String
   Return String.Empty
  End Function

  ''' <summary>
  ''' string XML representation.
  ''' </summary>
  ''' <returns>string</returns>
  Protected Shared Function ToXml(Of T As RssDocumentBase)(ByVal outputType As DocumentType, ByVal rssDocument As T) As String
   Dim rssXml As String = RssXmlHelper.ToRssXml(Of T)(rssDocument)

   Return RssXmlHelper.ConvertRssTo(outputType, rssXml)
  End Function

  ''' <summary>
  ''' Loads the specified URL.
  ''' </summary>
  ''' <param name="url">The URL.</param>
  ''' <returns>RssDocument</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId:="0#"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter")> _
  Protected Shared Function Load(Of T As RssDocumentBase)(ByVal url As System.Uri, ByVal tempDir As String, ByRef originalType As DocumentType) As T
   If url Is Nothing Then
    Throw New ArgumentNullException("url")
   End If

   ' resolve app-relative URLs
   Dim rssUrl As String = RssXmlHelper.ResolveAppRelativeLinkToUrl(url.ToString())

   ' download the feed
   Using cachedXml As Stream = DownloadManager.GetFeed(rssUrl, tempDir)
    Using reader As New XmlTextReader(cachedXml)
     Dim rss As T = Load(Of T)(reader, tempDir, originalType)

     '  remember the url
     rss.Url = rssUrl

     Return rss
    End Using
   End Using
  End Function

  ''' <summary>
  ''' Loads the specified reader.
  ''' </summary>
  ''' <param name="reader">The reader.</param>
  ''' <returns>Generic Type of RssDocumentBase</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter")> _
  Protected Shared Function Load(Of T As RssDocumentBase)(ByVal reader As XmlReader, ByVal tempDir As String, ByRef originalType As DocumentType) As T
   Dim rss As T = Nothing
   Dim rssXml As String = RssXmlHelper.ConvertToRssXml(reader, tempDir, originalType)

   If Not String.IsNullOrEmpty(rssXml) Then
    rss = LoadFromRssXml(Of T)(rssXml)
   End If

   Return rss
  End Function

  ''' <summary>
  ''' Loads from XML.
  ''' </summary>
  ''' <param name="xml">The XML.</param>
  ''' <returns>Generic Type of RssDocumentBase</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter")> _
  Protected Shared Function Load(Of T As RssDocumentBase)(ByVal xml As String, ByVal tempDir As String, ByRef originalType As DocumentType) As T
   Dim rssDocument As T = Nothing
   Using stringReader As New StringReader(xml)
    Using reader As New XmlTextReader(stringReader)
     rssDocument = Load(Of T)(reader, tempDir, originalType)
    End Using
   End Using

   Return rssDocument
  End Function

  Private Shared Function LoadFromRssXml(Of T As RssDocumentBase)(ByVal rssXml As String) As T
   If String.IsNullOrEmpty(rssXml) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "xml"))
   End If

   Dim rss As T = RssXmlHelper.DeserializeFromXmlUsingStringReader(Of T)(rssXml)

   Return rss
  End Function
 End Class
End Namespace
