'=======================================================================
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'  PARTICULAR PURPOSE.
'=======================================================================


Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Data
Imports System.Globalization
Imports System.IO
Imports System.Reflection
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Web
Imports System.Xml
Imports System.Xml.Xsl
Imports System.Xml.XPath
Imports System.Xml.Serialization

Namespace DotNetNuke.Modules.News.RssToolkit.Rss
 ''' <summary>
 ''' Helper class
 ''' </summary>
 ''' <history>
 ''' 	[pdonker]	01/03/2008  Caching directory no longer retrieved from web.config but passed as a parameter
 ''' 	[pdonker]	01/03/2008  Distinction between Rss 0.91 and 2.0
 ''' </history>
 Public NotInheritable Class RssXmlHelper
  Private Sub New()
  End Sub
  Private Const TimeZoneCacheKey As String = "DateTimeParser"

  ''' <summary>
  ''' Resolves the app relative link to URL.
  ''' </summary>
  ''' <param name="link">The link.</param>
  ''' <returns>string</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings")> _
  Public Shared Function ResolveAppRelativeLinkToUrl(ByVal link As String) As String
   If String.IsNullOrEmpty(link) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "link"))
   End If

   If Not String.IsNullOrEmpty(link) AndAlso link.StartsWith("~/") Then
    Dim context As HttpContext = HttpContext.Current

    If context IsNot Nothing Then
     Dim query As String = String.Empty
     Dim iquery As Integer = link.IndexOf("?"c)

     If iquery >= 0 Then
      query = link.Substring(iquery)
      link = link.Substring(0, iquery)
     End If

     link = VirtualPathUtility.ToAbsolute(link)
     link = New Uri(context.Request.Url, link).ToString() + query
    End If
   End If

   Return link
  End Function

  ''' <summary>
  ''' Deserialize from XML using string reader.
  ''' </summary>
  ''' <param name="xml">The XML.</param>
  ''' <typeparam name="T">RssDocumentBase</typeparam>
  ''' <returns>T</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter")> _
  Public Shared Function DeserializeFromXmlUsingStringReader(Of T As RssDocumentBase)(ByVal xml As String) As T
   If String.IsNullOrEmpty(xml) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "xml"))
   End If

   Dim serializer As New XmlSerializer(GetType(T))
   Using reader As New StringReader(xml)
    Return DirectCast(serializer.Deserialize(reader), T)
   End Using
  End Function

  ''' <summary>
  ''' Returns XML of the Generic Type.
  ''' </summary>
  ''' <param name="rssDocument">The RSS document.</param>
  ''' <typeparam name="T">RssDocumentBase</typeparam>
  ''' <returns>string</returns>
  Public Shared Function ToRssXml(Of T As RssDocumentBase)(ByVal rssDocument As T) As String
   If rssDocument Is Nothing Then
    Throw New ArgumentNullException("rssDocument")
   End If

   Using output As New StringWriter(New StringBuilder(), CultureInfo.InvariantCulture)
    Dim serializer As New XmlSerializer(GetType(T))
    serializer.Serialize(output, rssDocument)
    Return output.ToString()
   End Using
  End Function

  Public Shared Function GetDocumentType(ByVal input As String) As DocumentType
   Using stringReader As New StringReader(input)
    Using reader As New XmlTextReader(stringReader)
     While reader.Read()
      If reader.NodeType = XmlNodeType.Element Then
       Return GetDocumentType(reader)
      End If
     End While
    End Using
   End Using
  End Function

  ''' <summary>
  ''' Gets the type of the document.
  ''' </summary>
  ''' <returns>RssDocument</returns>
  Public Shared Function GetDocumentType(ByRef node As XmlReader) As DocumentType

   Dim nodeName As String = node.LocalName
   If String.IsNullOrEmpty(nodeName) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "nodeName"))
   End If

   If nodeName.Equals("rss", StringComparison.OrdinalIgnoreCase) Then
    For i As Integer = 0 To node.AttributeCount - 1
     Dim version As String = node.GetAttribute(i)
     Select Case version
      Case "2.0"
       Return DocumentType.Rss20
      Case "0.91"
       Return DocumentType.Rss091
     End Select
    Next
   ElseIf nodeName.Equals("opml", StringComparison.OrdinalIgnoreCase) Then
    Return DocumentType.Opml
   ElseIf nodeName.Equals("feed", StringComparison.OrdinalIgnoreCase) Then
    Return DocumentType.Atom
   ElseIf nodeName.Equals("rdf", StringComparison.OrdinalIgnoreCase) Then
    Return DocumentType.Rdf
   End If

   Return DocumentType.Unknown
  End Function

  ''' <summary>
  ''' Converts to RSS XML.
  ''' </summary>
  ''' <param name="input">The input.</param>
  ''' <returns>Xml string in Rss Format</returns>
  Public Shared Function ConvertToRssXml(ByVal input As String, ByVal tempDir As String, ByRef originalType As DocumentType) As String
   Using stringReader As New StringReader(input)
    Using reader As New XmlTextReader(stringReader)
     Return ConvertToRssXml(reader, tempDir, originalType)
    End Using
   End Using
  End Function

  ''' <summary>
  ''' Converts to RSS XML.
  ''' </summary>
  ''' <param name="reader">The reader.</param>
  ''' <returns>Xml string in Rss Format</returns>
  Public Shared Function ConvertToRssXml(ByVal reader As XmlReader, ByVal tempDir As String, ByRef originalType As DocumentType) As String
   If reader Is Nothing Then
    Throw New ArgumentNullException("reader")
   End If

   Dim rssXml As String = String.Empty

   While reader.Read()
    If reader.NodeType = XmlNodeType.Element Then
     originalType = RssXmlHelper.GetDocumentType(reader)
     Dim outerXml As String = reader.ReadOuterXml()
     Select Case originalType
      Case DocumentType.Rss20
       rssXml = outerXml
       Exit Select
      Case DocumentType.Rss091
       rssXml = DoXslTransform(outerXml, GetStreamFromResource(Constants.Rss091ToRssXsl))
       Exit Select
      Case DocumentType.Opml
       Dim aggregator As New RssAggregator(tempDir)
       aggregator.Load(outerXml)
       rssXml = aggregator.OuterXml
       Exit Select
      Case DocumentType.Atom
       rssXml = DoXslTransform(outerXml, GetStreamFromResource(Constants.AtomToRssXsl))
       Exit Select
      Case DocumentType.Rdf
       rssXml = DoXslTransform(outerXml, GetStreamFromResource(Constants.RdfToRssXsl))
       Exit Select
     End Select

     Exit While
    End If
   End While

   Return rssXml
  End Function

  ''' <summary>
  ''' Gets the data set.
  ''' </summary>
  ''' <param name="xml">XML</param>
  ''' <returns>DataSet</returns>
  Public Shared Function ToDataSet(ByVal xml As String) As DataSet
   If String.IsNullOrEmpty(xml) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "xml"))
   End If

   Dim dataset As New DataSet()
   dataset.Locale = CultureInfo.InvariantCulture
   Using stringReader As New StringReader(xml)
    dataset.ReadXml(stringReader)
   End Using

   Return dataset
  End Function

  ''' <summary>
  ''' Loads the RSS from opml URL.
  ''' </summary>
  ''' <param name="url">The URL.</param>
  ''' <typeparam name="T">RssDocumentBase</typeparam>
  ''' <returns>Generic of RssDocumentBase</returns>
  <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId:="0#")> _
  Public Shared Function LoadRssFromOpmlUrl(Of T As RssDocumentBase)(ByVal url As String, ByVal tempDir As String) As T
   If String.IsNullOrEmpty(url) Then
    Throw New ArgumentException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.ArgmentException, "url"))
   End If

   ' resolve app-relative URLs
   url = RssXmlHelper.ResolveAppRelativeLinkToUrl(url)

   Dim aggregator As New RssAggregator(tempDir)
   aggregator.Load(New System.Uri(url))
   Dim rssXml As String = aggregator.OuterXml

   Return RssXmlHelper.DeserializeFromXmlUsingStringReader(Of T)(rssXml)
  End Function

  ''' <summary>
  ''' Parse is able to parse RFC2822/RFC822 formatted dates.
  ''' It has a fallback mechanism: if the string does not match,
  ''' the normal DateTime.Parse() function is called.
  ''' 
  ''' Copyright of RssBandit.org
  ''' Author - t_rendelmann
  ''' </summary>
  ''' <param name="dateTime">Date Time to parse</param>
  ''' <returns>DateTime instance</returns>
  ''' <exception cref="FormatException">On format errors parsing the DateTime</exception>
  Public Shared Function Parse(ByVal dateTime As String) As DateTime
   Dim rfc2822 As New Regex("\s*(?:(?:Mon|Tue|Wed|Thu|Fri|Sat|Sun)\s*,\s*)?(\d{1,2})\s+(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s+(\d{2,})\s+(\d{2})\s*:\s*(\d{2})\s*(?::\s*(\d{2}))?\s+([+\-]\d{4}|UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT|[A-IK-Z])", RegexOptions.Compiled)
   Dim months As New ArrayList(New String() {"ZeroIndex", "Jan", "Feb", "Mar", "Apr", "May", _
    "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", _
    "Dec"})

   If dateTime Is Nothing Then
    Return System.DateTime.Now.ToUniversalTime()
   End If

   If dateTime.Trim().Length = 0 Then
    Return System.DateTime.Now.ToUniversalTime()
   End If

   Dim m As Match = rfc2822.Match(dateTime)
   If m.Success Then
    Try
     Dim dd As Integer = Int32.Parse(m.Groups(1).Value, CultureInfo.InvariantCulture)
     Dim mth As Integer = months.IndexOf(m.Groups(2).Value)
     Dim yy As Integer = Int32.Parse(m.Groups(3).Value, CultureInfo.InvariantCulture)
     '  following year completion is compliant with RFC 2822.
     yy = CInt((IIf(yy < 50, 2000 + yy, (IIf(yy < 1000, 1900 + yy, yy)))))
     Dim hh As Integer = Int32.Parse(m.Groups(4).Value, CultureInfo.InvariantCulture)
     Dim mm As Integer = Int32.Parse(m.Groups(5).Value, CultureInfo.InvariantCulture)
     Dim ss As Integer = Int32.Parse(m.Groups(6).Value, CultureInfo.InvariantCulture)
     Dim zone As String = m.Groups(7).Value

     Dim xd As New DateTime(yy, mth, dd, hh, mm, ss)
     Return xd.AddHours(RFCTimeZoneToGMTBias(zone) * -1)
    Catch e As FormatException
     Throw New FormatException(String.Format(Resources.RssToolkit.Culture, Resources.RssToolkit.RssText, e.[GetType]().Name), e)
    End Try
   Else
    ' fall-back, if regex does not match:
    Return System.DateTime.Parse(dateTime, CultureInfo.InvariantCulture)
   End If
  End Function

  ''' <summary>
  ''' Converts a DateTime to a valid RFC 2822/822 string
  ''' </summary>
  ''' <param name="dt">The DateTime to convert, recognizes Zulu/GMT</param>
  ''' <returns>Returns the local time in RFC format with the time offset properly appended</returns>
  Public Shared Function ToRfc822(ByVal dt As DateTime) As String
   Dim timeZone As String

   If dt.Kind = DateTimeKind.Utc Then
    timeZone = "Z"
   Else
    Dim offset As TimeSpan = System.TimeZone.CurrentTimeZone.GetUtcOffset(dt)

    If offset.Ticks < 0 Then
     offset = -offset
     timeZone = "-"
    Else
     timeZone = "+"
    End If

    timeZone += offset.Hours.ToString(CultureInfo.InvariantCulture).PadLeft(2, "0"c)
    timeZone += offset.Minutes.ToString(CultureInfo.InvariantCulture).PadLeft(2, "0"c)
   End If

   Return dt.ToString("ddd, dd MMM yyyy HH:mm:ss " + timeZone, CultureInfo.InvariantCulture)
  End Function

  ''' <summary>
  ''' Does the XSL transform.
  ''' </summary>
  ''' <param name="inputXml">The input XML.</param>
  ''' <param name="xslResource">The XSL resource.</param>
  ''' <returns></returns>
  Public Shared Function DoXslTransform(ByVal inputXml As String, ByVal xslResource As Stream) As String
   ' TODO Make the culture an argument
   Using outputWriter As New StringWriter(System.Threading.Thread.CurrentThread.CurrentUICulture)
    Using stringReader As New StringReader(inputXml)
     Dim xpathDoc As New XPathDocument(stringReader)
     Dim params As New XsltArgumentList
     params.AddParam("Now", "", DateTime.Now.ToString("u"))
     Dim transform As New XslCompiledTransform()
     Using styleSheetReader As New XmlTextReader(xslResource)
      transform.Load(styleSheetReader)
      transform.Transform(xpathDoc, params, outputWriter)
     End Using
    End Using

    Return outputWriter.ToString()
   End Using
  End Function

  ''' <summary>
  ''' Changes Time zone based on GMT
  ''' 
  ''' Copyright of RssBandit.org
  ''' Author - t_rendelmann
  ''' </summary>
  ''' <param name="zone">The zone.</param>
  ''' <returns>RFCTimeZoneToGMTBias</returns>
  Private Shared Function RFCTimeZoneToGMTBias(ByVal zone As String) As Double
   Dim timeZones As Dictionary(Of String, Integer) = Nothing

   If HttpContext.Current IsNot Nothing Then
    timeZones = TryCast(HttpContext.Current.Cache(TimeZoneCacheKey), Dictionary(Of String, Integer))
   End If

   If timeZones Is Nothing Then
    timeZones = New Dictionary(Of String, Integer)()
    timeZones.Add("GMT", 0)
    timeZones.Add("UT", 0)
    timeZones.Add("EST", -5 * 60)
    timeZones.Add("EDT", -4 * 60)
    timeZones.Add("CST", -6 * 60)
    timeZones.Add("CDT", -5 * 60)
    timeZones.Add("MST", -7 * 60)
    timeZones.Add("MDT", -6 * 60)
    timeZones.Add("PST", -8 * 60)
    timeZones.Add("PDT", -7 * 60)
    timeZones.Add("Z", 0)
    timeZones.Add("A", -1 * 60)
    timeZones.Add("B", -2 * 60)
    timeZones.Add("C", -3 * 60)
    timeZones.Add("D", -4 * 60)
    timeZones.Add("E", -5 * 60)
    timeZones.Add("F", -6 * 60)
    timeZones.Add("G", -7 * 60)
    timeZones.Add("H", -8 * 60)
    timeZones.Add("I", -9 * 60)
    timeZones.Add("K", -10 * 60)
    timeZones.Add("L", -11 * 60)
    timeZones.Add("M", -12 * 60)
    timeZones.Add("N", 1 * 60)
    timeZones.Add("O", 2 * 60)
    timeZones.Add("P", 3 * 60)
    timeZones.Add("Q", 4 * 60)
    timeZones.Add("R", 3 * 60)
    timeZones.Add("S", 6 * 60)
    timeZones.Add("T", 3 * 60)
    timeZones.Add("U", 8 * 60)
    timeZones.Add("V", 3 * 60)
    timeZones.Add("W", 10 * 60)
    timeZones.Add("X", 3 * 60)
    timeZones.Add("Y", 12 * 60)

    If HttpContext.Current IsNot Nothing Then
     HttpContext.Current.Cache.Insert(TimeZoneCacheKey, timeZones)
    End If
   End If

   If zone.IndexOfAny(New Char() {"+"c, "-"c}) = 0 Then
    ' +hhmm format
    Dim sign As Integer = CInt((IIf(zone(0) = "-"c, -1, 1)))
    Dim s As String = zone.Substring(1).TrimEnd()
    Dim hh As Integer = Math.Min(23, Int32.Parse(s.Substring(0, 2), CultureInfo.InvariantCulture))
    Dim mm As Integer = Math.Min(59, Int32.Parse(s.Substring(2, 2), CultureInfo.InvariantCulture))
    Return sign * (hh + (mm / 60))
   Else
    ' named format
    Dim s As String = zone.ToUpper(CultureInfo.InvariantCulture).Trim()
    For Each key As String In timeZones.Keys
     If key.Equals(s) Then
      Return timeZones(key) / 60
     End If
    Next
   End If

   Return 0
  End Function

  ''' <summary>
  ''' Converts the atom to RSS.
  ''' </summary>
  ''' <param name="ouputType">Rss Document Type</param>
  ''' <param name="rssXml">The input XML.</param>
  ''' <returns>string</returns>
  Public Shared Function ConvertRssTo(ByVal ouputType As DocumentType, ByVal rssXml As String) As String
   If String.IsNullOrEmpty(rssXml) Then
    Return Nothing
   End If

   Select Case ouputType
    Case DocumentType.Rss20
     Return rssXml
    Case DocumentType.Rss091
     Using stream As Stream = GetStreamFromResource(Constants.Rss091ToRssXsl)
      Return RssXmlHelper.DoXslTransform(rssXml, stream)
     End Using
    Case DocumentType.Atom
     Using stream As Stream = GetStreamFromResource(Constants.RssToAtomXsl)
      Return RssXmlHelper.DoXslTransform(rssXml, stream)
     End Using
    Case DocumentType.Rdf
     Using stream As Stream = GetStreamFromResource(Constants.RssToRdfXsl)
      Return RssXmlHelper.DoXslTransform(rssXml, stream)
     End Using
    Case DocumentType.Opml
     Using stream As Stream = GetStreamFromResource(Constants.RssToOpmlXsl)
      Return RssXmlHelper.DoXslTransform(rssXml, stream)
     End Using
    Case Else
     Return Nothing
   End Select
  End Function

  Private Shared Function GetStreamFromResource(ByVal resourceFileName As String) As Stream
   Dim assembly As Assembly = assembly.GetExecutingAssembly()
   If assembly IsNot Nothing Then
    Return assembly.GetManifestResourceStream(resourceFileName)
   End If

   Return Nothing
  End Function
 End Class
End Namespace
